#proto.spec -> base.spec -> mesa.spec -> server.spec

%define ALD_Ver		1.2.4

Name:         X11-base-2
Version:      7.7
Release:      4

License:      The Xorg Project
Summary:      Basic X11 package
Group:        Development/Libraries

Source:  X11R%{version}.%{release}.base-2.tar
Source1: CROSS-X11.sh

%define DestDir		/X11/%{version}-%{release}
# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mfpu=vfpv3-d16 -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1
#------------------ CROSS DIR --------------------
%define _prefix /usr/X11R%{version}
%define _libdir	%{_prefix}/%{_lib}
%define protoPrefix /var/tmp/X11R%{version}-%{release}/%{TO_ARCH}
%define protoBinDir %{protoPrefix}/bin
%define protoDir %{protoPrefix}%{_prefix}

%define _defaultdocdir	%{_prefix}/share/doc
%define _mandir %{_prefix}/man

%if %{_CROSS}

%if %{_os} == "mingw"
ERROR: target=%{_os}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/bin
%define target_  %{target}-
%define hosttarget --host=%{target}

%define cross_prefix	%{csysdir}%{_prefix}
%define cross_bindir	%{csysdir}%{_bindir}
%define cross_sbindir	%{csysdir}%{_sbindir}
%define cross_libdir	%{csysdir}%{_libdir}
%define cross_includedir %{csysdir}%{_includedir}
%define cross_datadir	%{csysdir}%{_datadir}
%define cross_infodir	%{csysdir}%{_infodir}
%define cross_mandir	%{csysdir}%{_mandir}
%define cross_defaultdocdir %{csysdir}%{_defaultdocdir}
%define cross_docdir	%{cross_defaultdocdir}/%{name}

%define CrossDestDir	/Cross

%define _srcrpmdir	%{_topdir}/SRPMS/tmp

%else  #_CROSS

%define csysdir		%{nil}
%define csysbindir	%{_prefix}/bin
%define CrossDestDir	%{nil}
%define _srcrpmdir	%{_topdir}/SRPMS/%{ALD_Ver}

%define target %{_build}
%define target_  %{nil}
%define hosttarget %{nil}

%endif # !_CROSS

%define __os_install_post %{nil}

%define docdir		%{_defaultdocdir}/%{name}
%define fontrootdir	%{_prefix}/lib/X11/fonts

# ------------------- description ------------------------------------
%description
Contains Xorg and some other programs
#-------------------------------
%package devel
Summary:      Libraries for the C compiler
Group:        Development/Libraries
Requires:     X11-proto >= 7.7
%description devel
These libraries are needed to compile C code.
#-------------------------------
%package devel-man3
Summary:      man3 for Development
Group:        Development/Libraries
%description devel-man3
man3 for Development
#-------------------------------
%package doc
Summary:      Doc for X11
Group:        Documentation
%description doc
doc for X11
#-------------------------------
%package doc-devel
Summary:      Doc build data for X11
Group:        Development/Libraries
%description doc-devel
Doc build data for X11
#-------------------------------
%package xkeystone
Summary:      x11-xserver-utils xkeystone
Group:        Applications/Languages
%description xkeystone
x11-xserver-utils xkeystone

#-------------------------------
%package %{TO_ARCH}
Summary:      %{name} Cross-%{TO_ARCH} Libraries
Group:        Development/Libraries
%description %{TO_ARCH}
%{name} Cross-%{TO_ARCH} Libraries
# --------------------------------------
%package %{TO_ARCH}-doc
Summary:      %{name} Cross-%{TO_ARCH} doc
Group:        Development/Libraries
%description %{TO_ARCH}-doc
%{name} Cross-%{TO_ARCH} doc

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n X11R%{version}.%{release}.base-2

# ------------------- Build ------------------------------------------
%Build
%if %{_CROSS}
PATH=%{csysbindir}:%{protoBinDir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
export PATH GCC_NoHostPath
%else
export PATH=%{protoBinDir}:$PATH:/opt/perl/bin
%endif

case `python -V 2>&1` in
  'Python 2.6.'*) TARGET_PYTHONDIR=/python2.6/site-packages ;;
  'Python 2.7.'*) TARGET_PYTHONDIR=/python2.7/site-packages ;;
  'Python 3.4.'*) TARGET_PYTHONDIR=/python3.4/site-packages ;;
  'Python 3.5.'*) TARGET_PYTHONDIR=/python3.5/site-packages ;;
  'Python 3.6.'*) TARGET_PYTHONDIR=/python3.6/site-packages ;;
  *) 
    echo "ERROR: no python"
    exit 1
  ;;
esac

%if %{_arch} == "i386"
OPTFLAGS="-O2 -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-O2 -fPIC -fno-asynchronous-unwind-tables"
%endif
%if %{_arch} == "arm"
%if %{_target_cpu} == "armv7l"
OPTFLAGS="-O2 -fPIC -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%else
OPTFLAGS="-O2 -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
%endif
%endif


TARGET_PRFIX=%{_prefix}
RPM_TMP_ROOT=$RPM_BUILD_ROOT/tmp
TARGET_INSTDIR=$RPM_BUILD_ROOT%{csysdir}%{_prefix}
TARGET_NAME=%{target_}
TARGET_LIB=%{_lib}
export TARGET_PRFIX RPM_TMP_ROOT TARGET_INSTDIR TARGET_PYTHONDIR TARGET_NAME TARGET_LIB

TARGET_ProtoDir=%{protoDir}
TARGET_CrossDir=%{csysdir}%{_prefix}
MKCROSSDIR=%{BUILD_LOCAL_CROSS}
export TARGET_ProtoDir TARGET_CrossDir MKCROSSDIR

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_TMP_ROOT


LD_LIBRARY_PATH="%{protoPrefix}%{_libdir}"
XKBCOMP="%{_prefix}/bin/xkbcomp"
FONT_DESTDIR="%{protoDir}"
X11_INCLUDES="-I%{protoPrefix}%{_includedir}"

export LD_LIBRARY_PATH XKBCOMP FONT_DESTDIR X11_INCLUDES

prefX11="--build=%{_build} %{hosttarget} \
	--prefix=%{_prefix} \
	--libdir=%{_libdir} \
	--libexecdir=%{_libdir} \
	--datarootdir=%{_libdir} \
	--mandir=%{_mandir} \
	--localstatedir=/var \
	--sysconfdir=/etc"


PKG_CONFIG_PATH="%{protoPrefix}%{_libdir}/pkgconfig:%{protoPrefix}/usr/lib/pkgconfig:%{protoPrefix}/usr/share/pkgconfig"

CFLAGS="$OPTFLAGS -D_GNU_SOURCE"
LDFLAGS="-L%{protoPrefix}%{_libdir}"
CXXFLAGS="$OPTFLAGS"
export CFLAGS LDFLAGS CXXFLAGS PKG_CONFIG_PATH

%if %{_CROSS}
export xorg_cv_malloc0_returns_null="yes"
%endif

%if 1
sf="xkeyboard-config-2.9"
tar -xaf $sf.tar.xz
cd $sf
  xf=`echo $sf | cut -d '-' -f 1`
  ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf --enable-xkbcomp-symlink --disable-nls &>Fconfigure_log.txt
  make &>Fmake_log.txt
  make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
  rm -fr $RPM_TMP_ROOT%{_prefix}/share/locale
  rm -f  $RPM_TMP_ROOT%{_prefix}/%{_lib}/*.la
  %SOURCE1 $sf
cd ..

for sf in libxkbfile-1.0.8 libXext-1.3.3  \
	libXt-1.1.4 libXrender-0.9.8 libXrandr-1.4.2 \
	libXxf86dga-1.1.4 libXxf86vm-1.1.3 libXfixes-5.0.1 libXdamage-1.1.4 \
	libXpm-3.5.11 libXinerama-1.1.3 libXcomposite-0.4.4 \
	libXft-2.3.2 libXcursor-1.1.14 \
	libXv-1.0.9 libXvMC-1.0.8 libXi-1.7.4 libXtst-1.2.2 \
	libXmu-1.1.2 libXaw-1.0.12; do
  tar -xaf $sf.tar.xz
  cd $sf
    xf=`echo $sf | cut -d '-' -f 1`
    ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf --disable-static &>Fconfigure_log.txt
    make -j 2 &>Fmake_log.txt
    make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
    rm -f $RPM_TMP_ROOT%{_libdir}/*.la
    %SOURCE1 $sf
  cd ..
done

%endif



%if 1

for sf in xbitmaps-1.1.1 setxkbmap-1.3.0 xkbcomp-1.2.4 xsetroot-1.1.0 mkfontdir-1.0.7 \
	mkfontscale-1.1.1 xinit-1.3.2 xset-1.2.3 xrandr-1.4.2; do
  tar -xaf $sf.tar.xz
  cd $sf
    xf=`echo $sf | cut -d '-' -f 1`
    ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf --disable-silent-rules &>Fconfigure_log.txt
    make -j 2 &>Fmake_log.txt
    make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
    %SOURCE1 $sf
  cd ..
done

for sf in font-util-1.3.0 font-alias-1.0.3; do
  tar -xaf $sf.tar.xz
  cd $sf
    xf=`echo $sf | cut -d '-' -f 1`
    ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf --with-fontrootdir=%{fontrootdir} &>Fconfigure_log.txt
    make -j 2 &>Fmake_log.txt
    make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
    %SOURCE1 $sf
  cd ..
done

sed -i s:"fontrootdir=":"fontrootdir=%{protoPrefix}": %{protoPrefix}%{_libdir}/pkgconfig/fontutil.pc

sf="bdftopcf-1.0.4"
tar -xaf $sf.tar.xz
cd $sf
  xf=`echo $sf | cut -d '-' -f 1`
  ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf &>Fconfigure_log.txt
  make -j 2 &>Fmake_log.txt
  make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
  %SOURCE1 $sf
cd ..

sf="libxshmfence-1.1"
tar -xaf $sf.tar.xz
cd $sf
  sed -i s:" @XPROTO_CFLAGS@":: xshmfence.pc.in
  xf=`echo $sf | cut -d '-' -f 1`
  ./configure $prefX11 --docdir=%{_defaultdocdir}/$xf --disable-shared --with-shared-memory-dir=/tmp &>Fconfigure_log.txt
  make V=1 &>Fmake_log.txt
  make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
  rm -f $RPM_TMP_ROOT%{_libdir}/*.la
  %SOURCE1 $sf
cd ..

%endif


%if 1
sf="xterm-278"
tar -xaf $sf.tar.xz
cd $sf
  ./configure --build=%{_build} %{hosttarget} \
	--prefix=%{_prefix} \
	--libdir=%{_libdir} \
	--libexecdir=%{_libdir} \
	--mandir=%{_mandir} \
	--without-utempter \
	--with-x \
	--disable-imake \
	--disable-freetype \
	--disable-i18n \
	--disable-tek4014 \
	--disable-vt52 \
	--enable-wide-chars \
	--x-includes=%{protoPrefix}%{_includedir} \
	--x-libraries=%{protoPrefix}%{_libdir} \
	--disable-rpath-hack \
	&>Fconfigure_log.txt
  make &>Fmake_log.txt
  make DESTDIR=$RPM_TMP_ROOT install &>FInstall_log.txt
  %SOURCE1 $sf
%if %{_lib} != "lib"
#  rm -rf $RPM_BUILD_ROOT%{cross_prefix}/lib/X11
  mv $RPM_BUILD_ROOT%{_prefix}/lib/X11/app-defaults $RPM_BUILD_ROOT%{_libdir}/X11/
%endif
cd ..
%endif

%if %{BUILD_LOCAL_CROSS}
rm -rf $RPM_BUILD_ROOT%{cross_bindir}
rm -rf $RPM_BUILD_ROOT%{cross_includedir}/X11/bitmaps
rm -rf $RPM_BUILD_ROOT%{cross_libdir}/X11
rm -rf $RPM_BUILD_ROOT%{cross_mandir}
rm -rf $RPM_BUILD_ROOT%{cross_datadir}/pixmaps
%if %{_lib} != "lib"
rm -rf $RPM_BUILD_ROOT%{csysdir}%{_prefix}/lib
%endif
%endif #%{BUILD_LOCAL_CROSS}

# ------------------- Install ----------------------------------------
%Install
%if !%{_CROSS}
if test -f /opt/aldccp/bin/packages_idx; then
  /opt/aldccp/bin/packages_idx -p %{name} -v %{version} -r %{DestDir} $RPM_BUILD_ROOT
fi
%endif


# ------------------- Clean ------------------------------------------
%Clean
DE=%{version}-%{release}.%{TO_ARCH}.rpm
DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{DestDir}
DEC=%{version}-%{release}.%{_build_cpu}.rpm
DDC=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
  mv %{name}-devel-$SE $DD/%{name}-devel-$DE
  mv %{name}-devel-man3-$SE $DD/%{name}-devel-man3-$DE
  mv %{name}-doc-$SE $DD/%{name}-doc-$DE
  mv %{name}-doc-devel-$SE $DD/%{name}-doc-devel-$DE
  mv %{name}-xkeystone-$SE $DD/%{name}-xkeystone-$DE
%if %{BUILD_LOCAL_CROSS}
  mkdir -p $DDC
  mv %{name}-%{TO_ARCH}-$SE $DDC/%{name}-%{TO_ARCH}-$DEC
  mv %{name}-%{TO_ARCH}-doc-$SE $DDC/%{name}-%{TO_ARCH}-doc-$DEC
%endif #BUILD_LOCAL_CROSS
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
  TMP_TO=$DD/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}.%{TO_ARCH}
  mkdir -p $tmp
  cp -r usr $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%if %{BUILD_LOCAL_CROSS}
  TMP_TO=$DDC/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{TO_ARCH}-%{version}-%{release}.%{_build_cpu}
  mkdir -p $tmp
  cp -r cross $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%endif
popd
%endif  #BUILD_TAR

# ------------------- Files ------------------------------------------
%Files
%{_bindir}
%exclude %{_bindir}/xkeystone
%{fontrootdir}
%{_libdir}/*.so.*
%{_libdir}/X11/app-defaults
%{_libdir}/X11/xinit
%{_libdir}/X11/xkb
%{_mandir}/man1
%{_mandir}/man7
%{_datadir}/pixmaps

%files devel
%{_includedir}
%{_libdir}/*.so
%{_libdir}/*.a
%{_libdir}/pkgconfig
/usr/share/aclocal

%files devel-man3
%{_mandir}/man3

%files doc
%{_defaultdocdir}/*/*.html
%{_defaultdocdir}/*/*.svg
%{_defaultdocdir}/*/*.txt

%files doc-devel
%{_defaultdocdir}/*/*.db
%{_defaultdocdir}/*/*.xml

%files xkeystone
%{_bindir}/xkeystone

# ------------------- Cross Files ------------------------------------
%if %{BUILD_LOCAL_CROSS}

%Files %{TO_ARCH}
%{cross_includedir}
%{cross_libdir}

%files %{TO_ARCH}-doc
%{cross_defaultdocdir}

%endif #!BUILD_LOCAL_CROSS
